
/*
** CREAPROC.SQL
**
** This script creates all the stored procedures for the Credit database.
**
** Don't forget to say 'USE Credit' first.
*/

PRINT 'Begin CREAPROC.SQL'
GO

USE Credit
SET NOCOUNT ON
GO
IF ( (SELECT db_name()) <> 'Credit')
BEGIN
   RAISERROR ('Database "Credit" not found.',22,127) WITH LOG
END
GO


CREATE PROCEDURE straight_rand 
    @max_value int
AS
BEGIN
   RETURN (rand() * @max_value) + 1
END
GO



CREATE PROCEDURE skewed_rand 
    @max_value int
  , @skew_factor tinyint
AS
BEGIN
   DECLARE @rand_value int    SELECT @rand_value = @max_value
   WHILE (@skew_factor > 0)
   BEGIN
      SELECT @skew_factor = @skew_factor - 1
      EXEC @rand_value = straight_rand @rand_value
   END
   RETURN (@max_value - @rand_value) + 1
END
GO



/*    
** If the objects already exist (i.e. if this is a rebuild), drop them.
*/

IF exists ( SELECT name FROM sysobjects
            WHERE type = 'P' and name = 'addcategory' )
    DROP PROCEDURE addcategory

IF exists ( SELECT name FROM sysobjects
            WHERE type = 'P' and name = 'addregion' )
    DROP PROCEDURE addregion

IF exists ( SELECT name FROM sysobjects
            WHERE type = 'P' and name = 'addprovider' )
    DROP PROCEDURE addprovider

IF exists ( SELECT name FROM sysobjects
            WHERE type = 'P' and name = 'addcorporation' )
    DROP PROCEDURE addcorporation

IF exists ( SELECT name FROM sysobjects
            WHERE type = 'P' and name = 'addmember' )
    DROP PROCEDURE addmember

IF exists ( SELECT name FROM sysobjects
            WHERE type = 'P' and name = 'removecategory' )
    DROP PROCEDURE removecategory

IF exists ( SELECT name FROM sysobjects
            WHERE type = 'P' and name = 'removeregion' )
    DROP PROCEDURE removeregion

IF exists ( SELECT name FROM sysobjects
            WHERE type = 'P' and name = 'removeprovider' )
    DROP PROCEDURE removeprovider

IF exists ( SELECT name FROM sysobjects
            WHERE type = 'P' and name = 'removecorporation' )
    DROP PROCEDURE removecorporation

IF exists ( SELECT name FROM sysobjects
            WHERE type = 'P' and name = 'removemember' )
    DROP PROCEDURE removemember

IF exists ( SELECT name FROM sysobjects
            WHERE type = 'P' and name = 'inputcharge' )
    DROP PROCEDURE inputcharge

IF exists ( SELECT name FROM sysobjects
            WHERE type = 'P' and name = 'generatestatement' )
    DROP PROCEDURE generatestatement

IF exists ( SELECT name FROM sysobjects
            WHERE type = 'P' and name = 'inputpayment' )
    DROP PROCEDURE inputpayment
GO


CREATE PROCEDURE addcategory
    @category_desc    shortstring
AS 
    INSERT category
        (  category_desc)
      VALUES    
        ( @category_desc)

    IF @@error != 0
       RETURN (-99)
    ELSE
       RETURN 0
GO


CREATE PROCEDURE addregion
    @region_name      shortstring
  , @street           shortstring
  , @city             shortstring
  , @state_prov       statecode
  , @country          countrycode
  , @mail_code        mailcode
  , @phone_no         phonenumber 
AS 
    INSERT region
        (  region_name,  street,  city,  state_prov,  country,  mail_code,  phone_no)
      VALUES    
        ( @region_name, @street, @city, @state_prov, @country, @mail_code, @phone_no)

    IF @@error != 0
       RETURN (-99)
    ELSE
       RETURN 0
GO


CREATE PROCEDURE addprovider
    @region_no        numeric_id
  , @provider_name    shortstring
  , @street           shortstring
  , @city             shortstring
  , @state_prov       statecode
  , @country          countrycode
  , @mail_code        mailcode
  , @phone_no         phonenumber 
AS 
    INSERT provider
        (  region_no,  provider_name,  street,  city,  state_prov,  country,  mail_code,  phone_no)
      VALUES    
        ( @region_no, @provider_name, @street, @city, @state_prov, @country, @mail_code, @phone_no)

    IF @@error != 0
       RETURN (-99)
    ELSE
       RETURN 0
GO


CREATE PROCEDURE addcorporation
    @region_no        numeric_id
  , @corp_name        normstring
  , @street           shortstring
  , @city             shortstring
  , @state_prov       statecode
  , @country          countrycode
  , @mail_code        mailcode
  , @phone_no         phonenumber 
  , @expr_dt          datetime
AS 
    INSERT corporation
        (  region_no,  corp_name,  street,  city,  state_prov,  country,  mail_code,  phone_no,  expr_dt)
      VALUES    
        ( @region_no, @corp_name, @street, @city, @state_prov, @country, @mail_code, @phone_no, @expr_dt)

    IF @@error != 0
       RETURN (-99)
    ELSE
       RETURN 0
GO



CREATE PROCEDURE addmember
    @region_no        numeric_id
  , @corp_no          numeric_id
  , @lastname         shortstring
  , @firstname        shortstring
  , @middleinitial    shortstring
  , @street           shortstring
  , @city             shortstring
  , @state_prov       statecode
  , @country          countrycode
  , @mail_code        mailcode
  , @phone_no         phonenumber 
AS 
    INSERT member
        (  region_no,  corp_no,  lastname,  firstname,  middleinitial,  street,  city,  state_prov,  country,  mail_code,  phone_no)
      VALUES    
        ( @region_no, @corp_no, @lastname, @firstname, @middleinitial, @street, @city, @state_prov, @country, @mail_code, @phone_no)

    IF @@error != 0
       RETURN (-99)
    ELSE
       RETURN 0
GO


CREATE PROCEDURE removecategory
    @category_no    numeric_id
AS 
    DELETE category
      WHERE category_no = @category_no

    IF @@error != 0
       RETURN (-99)
    ELSE
       RETURN 0
GO


CREATE PROCEDURE removeregion
    @region_no    numeric_id
AS 
    DELETE region
      WHERE region_no = @region_no

    IF @@error != 0
       RETURN (-99)
    ELSE
       RETURN 0
GO


CREATE PROCEDURE removeprovider
    @provider_no    numeric_id
AS 
    DELETE provider
      WHERE provider_no = @provider_no

    IF @@error != 0
       RETURN (-99)
    ELSE
       RETURN 0
GO


CREATE PROCEDURE removecorporation
    @corp_no    numeric_id
AS 
    DELETE corporation
      WHERE corp_no = @corp_no

    IF @@error != 0
       RETURN (-99)
    ELSE
       RETURN 0
GO


CREATE PROCEDURE removemember
    @member_no    numeric_id
AS 
    DELETE member
      WHERE member_no = @member_no

    IF @@error != 0
       RETURN (-99)
    ELSE
       RETURN 0
GO


CREATE PROCEDURE generatestatement 
    @member_no numeric_id
  , @statement_dt datetime  
AS
BEGIN
   DECLARE @due_dt datetime        SELECT @due_dt = DATEADD(day,20,@statement_dt)
   DECLARE @statement_amt     money
   DECLARE @statement_current money
   DECLARE @statement_pastdue money
   DECLARE @statement_latechg money
   DECLARE @factor int

   BEGIN TRANSACTION

   SELECT @statement_amt = ISNULL(SUM(charge_amt),0)
     FROM charge (TABLOCKX HOLDLOCK)
    WHERE member_no = @member_no 
      AND statement_no = 0
      AND charge_dt <= @statement_dt

   EXEC @factor = skewed_rand 100, 3
   SELECT @statement_current = (@statement_amt * @factor) / 100
   SELECT @statement_pastdue = @statement_amt - @statement_current
   SELECT @statement_latechg = @statement_pastdue * .10
   SELECT @statement_amt = @statement_current + @statement_pastdue + @statement_latechg

   INSERT statement (member_no        
                  ,  statement_dt     
                  ,  due_dt           
                  ,  statement_amt    
                  ,  statement_current    
                  ,  statement_pastdue    
                  ,  statement_latechg    
                  ,  statement_code
                  )
   VALUES (@member_no
        ,  @statement_dt
        ,  @due_dt
        ,  @statement_amt
        ,  @statement_current
        ,  @statement_pastdue
        ,  @statement_latechg
        ,  ' '
        )

   UPDATE charge
   SET statement_no = @@IDENTITY
   WHERE member_no = @member_no 
     AND statement_no = 0
     AND charge_dt <= @statement_dt

   COMMIT TRANSACTION
END
GO


CREATE PROCEDURE inputpayment
    @member_no          numeric_id   
  , @payment_dt         datetime     
  , @payment_amt        money        
  , @payment_principle  money        
  , @payment_latechg    money        
WITH RECOMPILE
AS 
BEGIN
  DECLARE @gap int  
  DECLARE @payment_due_dt datetime

  EXEC @gap = straight_rand 21  SELECT @gap = @gap - 7
  SELECT @payment_due_dt = dateadd(day,@gap,@payment_dt)

  BEGIN TRANSACTION
  IF @@error = 0
    INSERT payment
        (  member_no,  payment_dt,  payment_due_dt,  payment_amt,  payment_principle,  payment_latechg )
      VALUES    
        ( @member_no, @payment_dt, @payment_due_dt, @payment_amt, @payment_principle, @payment_latechg )
  IF @@error = 0
    COMMIT TRANSACTION
  IF @@error = 0
    RETURN 0

  ROLLBACK TRANSACTION
  RAISERROR ('Stored procedure "inputpayment" failed', 16, -1)
  RETURN (-99)
END
GO


CREATE PROCEDURE inputcharge
    @member_no        numeric_id   
  , @provider_no      numeric_id   
  , @category_no      numeric_id   
  , @charge_dt        datetime     
  , @charge_amt       money        
WITH RECOMPILE
AS 
BEGIN
  BEGIN TRANSACTION
  IF @@error = 0
    INSERT charge 
        (  member_no,  provider_no,  category_no,  charge_dt,  charge_amt )
      VALUES    
        ( @member_no, @provider_no, @category_no, @charge_dt, @charge_amt )
  IF @@error = 0
    COMMIT TRANSACTION
  IF @@error = 0
    RETURN 0

  ROLLBACK TRANSACTION
  RAISERROR ('Stored procedure "inputcharge" failed', 16, -1)
  RETURN (-99)
END
GO


